@@ -7,11 +7,24 @@ from requests import Session
 from urllib.parse import urljoin
 from .authenticate import Authenticate
 
+
 class Runner(Authenticate):
-    def __init__(self, action_name, asset_name='asset'):
-        # Load action/asset data
-        self.asset_manifest, self.action_manifest = self._load_schemas(action_name, asset_name)
-        self.asset, self.inputs = self._load_data()
+    def __init__(self, asset_host_key='host', asset_name='asset', action_name=None, context=None):
+        self.asset_host_key = asset_host_key
+        # Swimlane
+        if context:
+            self.asset = context.asset
+            self.inputs = context.inputs
+            self.endpoint = context.manifest['meta']['endpoint']
+            self.method = context.manifest['meta']['method']
+            self.security = context.manifest['meta']['security']
+        # Turbine
+        else:
+            self.asset_manifest, self.action_manifest = self._load_schemas(action_name, asset_name)
+            self.asset, self.inputs = self._load_data()
+            self.endpoint = self.action_manifest['meta']['endpoint']
+            self.method = self.action_manifest['meta']['method']
+            self.security = self.asset_manifest['meta']['security']
 
         # Build session
         self.session = Session()
@@ -23,7 +36,7 @@ class Runner(Authenticate):
         self.session.verify = os.getenv('verify') or self.asset.get('verify', True)
         self.raise_for_status = os.getenv('raise_for_status') or self.asset.get('raise_for_status', True)
 
-        super(Runner, self).__init__(self.session, self.asset, self.asset_manifest)
+        super(Runner, self).__init__(session=self.session, asset=self.asset, security=self.security)
 
     @staticmethod
     def _load_schemas(action_name, asset_name):
@@ -61,18 +74,18 @@ class Runner(Authenticate):
 
     def get_endpoint(self):
         # Mustache if available if not returns string as is.
-        return chevron.render(self.action_manifest['meta']['endpoint'], self.inputs.get('path_parameters', {}))
+        return chevron.render(self.endpoint, self.inputs.get('path_parameters', {}))
 
     def get_kwargs(self):
         # todo can we handle files automatically.
+        self.params.update(self.inputs.get('query_parameters'))
         return {
-            'params': self.inputs.get('query_parameters'),
+            'params': self.params,
+            'data': self.inputs.get('data_body'),
             'json': self.inputs.get('json_body'),
         }
 
     def parse_response(self, response):
-        if self.raise_for_status:
-            response.raise_for_status()
         try:
             json_body = response.json()
         except:
@@ -83,16 +96,18 @@ class Runner(Authenticate):
 
         return {
             'status_code': response.status_code,
-            'response_headers': response.headers,
+            'response_headers': dict(response.headers),
             'data': json_body,
             'response_text': response.text,
-            'raw_response': response.raw
+            'reason': response.reason
         }
 
     def run(self):
         response = self.session.request(
-            self.action_manifest['meta']['method'],
-            urljoin(self.asset['server'], self.get_endpoint()),
+            self.method,
+            urljoin(self.asset[self.asset_host_key], self.get_endpoint()),
             **self.get_kwargs()
         )
-        return self.parse_response(response)
+        resp = self.parse_response(response)
+        print('::set-output', json.dumps(resp))
+        return resp
